//   Copyright 2012 Georg-August-Universität Göttingen, Germany
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package de.ugoe.cs.autoquest.usability.rules.patterns.visitors.startswith;

import static de.ugoe.cs.autoquest.usability.tasktree.filters.EventTypeFilter.MOUSE_CLICK;
import static de.ugoe.cs.autoquest.usability.tasktree.filters.EventTypeFilter.TEXT_INPUT;
import static de.ugoe.cs.autoquest.usability.tasktree.filters.TaskTypeFilter.ITERATION;
import static de.ugoe.cs.autoquest.usability.tasktree.filters.TaskTypeFilter.SELECTION;
import static de.ugoe.cs.autoquest.usability.tasktree.filters.TaskTypeFilter.SEQUENCE;
import static org.fest.assertions.api.Assertions.assertThat;

import org.junit.Test;

import de.ugoe.cs.autoquest.tasktrees.treeifc.ITaskModel;
import de.ugoe.cs.autoquest.usability.rules.patterns.InteractionPattern;
import de.ugoe.cs.autoquest.usability.rules.patterns.InteractionPatternBuilder;
import de.ugoe.cs.autoquest.usability.testutil.GenerateTaskModelUtil;

/**
 * <p>
 * TODO comment
 * </p>
 * 
 * @author Alexander Deicke
 */
public class StartsWithPatternAndContainsPatternTest {

    @Test
    public void starts_with_pattern_and_contains_pattern_positive() {
        // Given
        String spec = "UserSession {" +
        		      "  Sequence seq1 {" +
                      "    Iteration iter1 {" +
                      "      MouseClick t1 {}" +
                      "    }" +
                      "    TextInput t2 {}" +
                      "    Selection sel1 {" +
                      "      MouseClick t3 {}" +
                      "    }" +
                      "    Selection sel1 {" +
                      "      Sequence seq2 {" +
                      "        TextInput t2 {}" +
                      "        MouseClick t3 {}" +
                      "      }" +
                      "    }" +
                      "  }" +
                      "  Iteration iter2 () {" +
                      "    MouseClick t1 {}" +
                      "  }" +
                      "}";
        ITaskModel taskTree = GenerateTaskModelUtil.getTaskModelFromSpec(spec);
        InteractionPatternBuilder patternBuilder = new InteractionPatternBuilder();
        InteractionPattern startsWithPattern = patternBuilder.concernedNode(ITERATION).contains(MOUSE_CLICK).build();
        InteractionPattern containedPattern = patternBuilder.concernedNode(SELECTION).startsWith(TEXT_INPUT).endsWith(MOUSE_CLICK).build();
        InteractionPattern pattern = patternBuilder.concernedNode(SEQUENCE).startsWith(startsWithPattern).contains(containedPattern).build();
        // When
        boolean patternContained = pattern.containedIn(taskTree);
        // Then
        assertThat(patternContained).isTrue();
    }
    
    @Test
    public void starts_with_pattern_and_contains_pattern_negative() {
        // Given
        String spec = "UserSession {" +
        		      "  Sequence seq1 {" +
                      "    Iteration iter1 {" +
                      "      MouseClick t1 {}" +
                      "    }" +
                      "    TextInput t2 {}" +
                      "    Selection sel1 {" +
                      "      MouseClick t3 {}" +
                      "    }" +
                      "    Selection sel1 {" +
                      "      Sequence seq2 {" +
                      "        MouseClick t2 {}" +
                      "        TextInput t3 {}" +
                      "      }" +
                      "    }" +
                      "  }" +
                      "  Iteration iter2 () {" +
                      "    MouseClick t1 {}" +
                      "  }" +
                      "}";
        ITaskModel taskTree = GenerateTaskModelUtil.getTaskModelFromSpec(spec);
        InteractionPatternBuilder patternBuilder = new InteractionPatternBuilder();
        InteractionPattern startsWithPattern = patternBuilder.concernedNode(ITERATION).contains(MOUSE_CLICK).build();
        InteractionPattern endsWithPattern = patternBuilder.concernedNode(SELECTION).startsWith(TEXT_INPUT).endsWith(MOUSE_CLICK).build();
        InteractionPattern pattern = patternBuilder.concernedNode(ITERATION).startsWith(endsWithPattern).endsWith(startsWithPattern).build();
        // When
        boolean patternContained = pattern.containedIn(taskTree);
        // Then
        assertThat(patternContained).isFalse();
    }
    
}
